Package: velociraptor
Title: Toolkit for Single-Cell Velocity
Version: 1.23.0
Date: 2026-04-15
Authors@R: c(person("Kevin", "Rue-Albrecht", role = c("aut", "cre"), email = "kevinrue67@gmail.com", comment = c(ORCID = "0000-0003-3899-3872")),
    person("Aaron", "Lun", role="aut", email="infinite.monkeys.with.keyboards@gmail.com", comment = c(ORCID = '0000-0002-3564-4813')),
    person("Charlotte", "Soneson", role="aut", email="charlottesoneson@gmail.com", comment = c(ORCID = '0000-0003-3833-2169')),
    person("Michael", "Stadler", role="aut", email="michael.stadler@fmi.ch", comment = c(ORCID = '0000-0002-2269-4934')))
Description: This package provides Bioconductor-friendly wrappers for RNA velocity calculations in single-cell RNA-seq data.
    We use the basilisk package to manage Conda environments,
    and the zellkonverter package to convert data structures between SingleCellExperiment (R) and AnnData (Python).
    The information produced by the velocity methods is stored in the various components of the SingleCellExperiment class.
Depends: SummarizedExperiment
Imports: methods, stats, Matrix, BiocGenerics, reticulate, S4Vectors,
        DelayedArray, basilisk, zellkonverter, scuttle,
        SingleCellExperiment, BiocParallel, BiocSingular
Suggests: BiocStyle, testthat, knitr, rmarkdown, pkgdown, scran,
        scater, scRNAseq, Rtsne, graphics, grDevices, ggplot2, cowplot,
        GGally, patchwork, metR
StagedInstall: no
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/kevinrue/velociraptor
BugReports: https://github.com/kevinrue/velociraptor/issues
biocViews: SingleCell, GeneExpression, Sequencing, Coverage
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/velociraptor
git_branch: devel
git_last_commit: 280d050
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-30 01:04:42 UTC; biocbuild
Author: Kevin Rue-Albrecht [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3899-3872>),
  Aaron Lun [aut] (ORCID: <https://orcid.org/0000-0002-3564-4813>),
  Charlotte Soneson [aut] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Michael Stadler [aut] (ORCID: <https://orcid.org/0000-0002-2269-4934>)
Maintainer: Kevin Rue-Albrecht <kevinrue67@gmail.com>
